/*
Copyright © 2013 Stefan Beller

This file is part of FLARE.

FLARE is free software: you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation,
either version 3 of the License, or (at your option) any later version.

FLARE is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with
FLARE.  If not, see http://www.gnu.org/licenses/
*/

#include "SharedGameResources.h"

MenuPowers *menu_powers = NULL;
LootManager *loot = NULL;
EnemyGroupManager *enemyg = NULL;
PowerManager *powers = NULL;
MapRenderer *mapr = NULL;
EnemyManager *enemies = NULL;
CampaignManager *camp = NULL;
ItemManager *items = NULL;
Avatar *pc = NULL;

