//
// The resource header (defines the icon).
//
#include "resource.h"

//
// Version information requires inclusion of either <windows.h> or <winver.h>
//
#include <WinVer.h>

// -------------------------------
//
// Icon
//

// Icon with lowest ID value placed first to ensure application icon
// remains consistent on all systems.
IDI_FLARE_APP_ICON        ICON                    "..\\distribution\\Flare.ico"

// -------------------------------
//
// Version information 
//

VS_VERSION_INFO VERSIONINFO
  FILEVERSION    0,15,0,0
  PRODUCTVERSION 0,15,0,0
BEGIN
    BLOCK "StringFileInfo"
    BEGIN
        BLOCK "040904E4"
        BEGIN
            VALUE "Comments", "Flare Alpha\0"
            VALUE "FileDescription", "Free Libre Action Roleplaying Engine\0"
            VALUE "FileVersion", "0, 15, 0\0"
            VALUE "InternalName", "Flare\0"
            VALUE "LegalCopyright", "Clint Bellanger and Contributors - 2011\0"
            VALUE "OriginalFilename", "flare.exe\0"
            VALUE "ProductName", "Flare\0"
            VALUE "ProductVersion", "0, 15, 0\0"
        END
    END
    BLOCK "VarFileInfo"
    BEGIN
		VALUE "Translation", 0x000, 1200
    END
END
